@@ -137,7 +137,7 @@ int rpl_lstat (const char *, struct stat *);
 # define FTW_NAME ftw
 # define NFTW_NAME nftw
 # define INO_T ino_t
-# define STAT stat
+# define FTW_STAT stat
 # ifdef _LIBC
 #  define LXSTAT __lxstat
 #  define XSTAT __xstat
@@ -157,6 +157,22 @@ int rpl_lstat (const char *, struct stat *);
 # define PATH_MAX 1024
 #endif
 
+#ifndef S_IFMT
+# define S_IFMT 0170000
+#endif
+
+#if STAT_MACROS_BROKEN
+# undef S_ISLNK
+#endif
+
+#ifndef S_ISLNK
+# ifdef S_IFLNK
+#  define S_ISLNK(m) (((m) & S_IFMT) == S_IFLNK)
+# else
+#  define S_ISLNK(m) 0
+# endif
+#endif
+
 struct dir_data
 {
   DIR *stream;
@@ -218,7 +234,8 @@ static const int ftw_arr[] =
 
 
 /* Forward declarations of local functions.  */
-static int ftw_dir (struct ftw_data *data, struct STAT *st) internal_function;
+static int ftw_dir (struct ftw_data *data, struct FTW_STAT *st)
+     internal_function;
 
 
 static int
@@ -237,7 +254,7 @@ object_compare (const void *p1, const void *p2)
 
 
 static inline int
-add_object (struct ftw_data *data, struct STAT *st)
+add_object (struct ftw_data *data, struct FTW_STAT *st)
 {
   struct known_object *newp = malloc (sizeof (struct known_object));
   if (newp == NULL)
@@ -249,7 +266,7 @@ add_object (struct ftw_data *data, struct STAT *st)
 
 
 static inline int
-find_object (struct ftw_data *data, struct STAT *st)
+find_object (struct ftw_data *data, struct FTW_STAT *st)
 {
   struct known_object obj;
   obj.dev = st->st_dev;
@@ -353,7 +370,7 @@ static inline int
 process_entry (struct ftw_data *data, struct dir_data *dir, const char *name,
 	       size_t namlen)
 {
-  struct STAT st;
+  struct FTW_STAT st;
   int result = 0;
   int flag = 0;
   size_t new_buflen;
@@ -465,7 +482,7 @@ process_entry (struct ftw_data *data, struct dir_data *dir, const char *name,
 
 static int
 internal_function
-ftw_dir (struct ftw_data *data, struct STAT *st)
+ftw_dir (struct ftw_data *data, struct FTW_STAT *st)
 {
   struct dir_data dir;
   struct dirent64 *d;
@@ -639,7 +656,7 @@ ftw_startup (const char *dir, int is_nftw, NFTW_FUNC_T func, int descriptors,
 	     int flags)
 {
   struct ftw_data data;
-  struct STAT st;
+  struct FTW_STAT st;
   int result = 0;
   int save_err;
   struct saved_cwd cwd;
