@@ -1,3 +1,20 @@
+/* Exit with a status code indicating failure.
+   Copyright (C) 1999-2003 Free Software Foundation, Inc.
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+
 #include <config.h>
 #include <stdio.h>
 #include <sys/types.h>
@@ -6,7 +23,7 @@
 #include "closeout.h"
 
 #define PROGRAM_NAME "false"
-#define AUTHORS "no one"
+#define AUTHORS "Jim Meyering"
 
 /* The name this program was run with. */
 char *program_name;
@@ -44,7 +61,7 @@ main (int argc, char **argv)
   if (argc == 2 && getenv ("POSIXLY_CORRECT") == NULL)
     {
       if (STREQ (argv[1], "--help"))
-	usage (EXIT_SUCCESS);
+	usage (EXIT_FAILURE);
 
       if (STREQ (argv[1], "--version"))
 	version_etc (stdout, PROGRAM_NAME, GNU_PACKAGE, VERSION, AUTHORS);
